#!/usr/bin/perl -w

use strict;
use warnings;
use IO::Socket::Multicast;
use constant GROUP => '224.199.71.138';
use constant LISTEN_PORT  => '6776';
use constant DESTINATION => '224.199.71.138:6776';

if ($#ARGV == 0 && $ARGV[0] =~ /-h/)
{
  print "usage: disco [-h] [-s] [-t timeout] [command [_IP_]]\n";
  print "Lists IP addresses or runs command for all Taurus's discovered on network.\n";
  print "where -h -- print this help and exit\n";
  print "      -s -- execute 'command' serially, not in parallel\n";
  print "      -t -- wait 'timeout' seconds for Taurus's to respond (default: 5)\n";
  print "      command -- command to execute for each Taurus found\n";
  print "      _IP_ -- replaced with actual Taurus IP address\n";
  print "      _SN_ -- replaced with actual Taurus serial number\n";
  exit;
}

my $commandSuffix = " &";
my $timeoutValue = "5";
while ($#ARGV >= 0 && $ARGV[0] =~ /^-/)
{
  if ($ARGV[0] =~ /-s/)
  {
    $commandSuffix = "";
    shift @ARGV;
    print "Executing commands serially.\n";
  }
  elsif ($ARGV[0] =~ /-t/)
  {
    shift @ARGV;
    if ($#ARGV < 0)
    {
      die "Missing timeout value for timeout option (-t timeout)\n";
    }
    $timeoutValue = $ARGV[0];
    shift @ARGV;
    print "Using timeout value of $timeoutValue seconds.\n";
  }
  else
  {
    die "Unrecognized option: $ARGV[0]\n";
  }
}

my $commandLine = "";
while($#ARGV >= 0)
{
  $commandLine .= shift(@ARGV) . " ";
}

# listen for replies
my $listenSock = IO::Socket::Multicast->new(LocalPort=>LISTEN_PORT);
$listenSock->mcast_add(GROUP) || die "Couldn't set group: $!\n";

# send query
my $sendSock = IO::Socket::Multicast->new(ReuseAddr=>1);
my $message = pack("N N", 1, -1);
$sendSock->mcast_send($message, DESTINATION) || die "Couldn't send: $!";

my $timedOut = 0;
$SIG{ALRM} = sub { $timedOut = 1; };
my $count = 0;
while(1)
{
  my $data;
  my $ip;
  alarm($timeoutValue);
  eval { $ip = $listenSock->recv($data, 8)};
  alarm(0);
  if (defined $ip)
  {
    my ($type, $id) = unpack("N N", $data);
    my $sn = $id & 0xffff;
    my $instrument = $id >> 16;
    if ($type == 3 && $instrument == 11)
    {
      $count ++;
      my ($port, $iaddr) = unpack_sockaddr_in($ip);
      my $ipName = inet_ntoa($iaddr);
      if ($commandLine eq "")
      {
        print "Found Taurus SN: $sn, with IP: $ipName\n";
      }
      else
      {
        my $theCommand = $commandLine . $commandSuffix;
        $theCommand =~ s/_IP_/$ipName/;
        $theCommand =~ s/_SN_/$sn/;
        system($theCommand);
      }
    }
  }
  else
  {
    if ($timedOut == 0)
    {
      print "Exiting after network error: $!\n";
    }
    else
    {
      print "Done - Taurus's found: $count\n";
    }
    last;
  }
}
