#!/usr/bin/perl
# =========================
# Instructions:
# 1) Make sure Perl is installed.  If Perl is not download it.  
# For Windows download from:
# http://www.activestate.com/Products/ActivePerl/?_x=1 (Download the MSI package)
# For Linux/Solaris download from: http://www.perl.org
#
# 2) To run the script from a command line/shell prompt type:
# perl extract_from_triggers.pl ip_address serial_number csv_file_name pre_tigger_time post_trigger_time
# 
# where ip_address is the ip_address of the machine running apollo (can use localhost for ApolloLite)
# serial_number is the serial_number of the unit
# csv_file_name is the name of the csv file containing the trigger
# pre_trigger_time is how many seconds before the trigger to download
# post_trigger_time is how many seconds after the trigger to download
# =========================
package NMX::Scripts;

use Time::Local;
use URI::URL;
use LWP::UserAgent;
use HTTP::Request;
use HTTP::Response;

# =========================
sub durationInMillis
{
    ( $duration ) = @_;
    return $duration * 1000;
}

# =========================
sub nanosToMillis
{
    ( $duration ) = @_;
    return $duration / 1000000;
}


# =========================
sub getChannel
{
    ( $serialNumber, $channelnum ) = @_;
    $channel = sprintf ("taurus_%04d/band/timeSeries%d/", $serialNumber, $channelnum);
    return $channel;
}

# =========================
sub getURL
{
    ( $ip_address, $startTime, $duration, $serialNumber ) = @_;
    my $channel1 = getChannel($serialNumber, 1);
    my $channel2 = getChannel($serialNumber, 2);
    my $channel3 = getChannel($serialNumber, 3);
    my $urlString =
    sprintf("http://%s/playback/download/?channels=%s,%s,%s&dataType=TimeSeries&dataFormat=MiniSEED&startMillis=%s&duration=%d s", $ip_address, $channel1, $channel2, $channel3, $startTime, $duration / 1000);
    $url = URI::URL->new($urlString);
    return $url;
}
# =========================
sub getTimeString
{
    ( $inputTime ) = @_;
    $inputTime =~ s/(^\s*|\s*$)//g;
    $seconds = $inputTime / 1000;
    $fraction += $inputTime % 1000;
    $fraction /= 1000;
    $units = "ms";
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = gmtime($seconds);
    $mon++;
    $year += 1900;
    my $date = sprintf( "%02d-%02d-%02d_%02d_%02d_%02d",
        $year, $mon, $mday, $hour, $min, $sec );
    return $date;
}

# =========================
sub getFileName
{
    ( $serialNumber, $triggerTime ) = @_;
    my $timeString = getTimeString($triggerTime);
    my $fileName = sprintf("taurus_%04d_trigger_%s.seed", $serialNumber,
    $timeString);
    return $fileName;
}
  
# =========================
sub trim
{
	( $string  ) = @_;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

# =========================
sub getColumnNumber
{
    ( $line ) = @_;

    my $columnName = "Start Time(ns)";
    my @columns = split(/,/, $line);
    for ($i = 0; $i <= $#columns; $i++) {
        my $column = trim($columns[$i]);
        if ($column eq $columnName){
            return $i; 
        }
    }
    my $msg = sprintf("Column <%s> was not found, aborting.\n", $columnName);
    die $msg;
}

# =========================
sub getFiles 
{
    ( $ip_address, $serialNumber, $csvFile, $preTime, $postTime ) = @_;
     
    open(my $in, $csvFile ) or die "Could not read $csvFile: $!";
    my $columnNumber = -1;
    while(<$in>){
        my $line = $_;
        if ($. == 1) {
           $columnNumber = getColumnNumber($line);
        } else {
            my @columns = split(/,/, $line);
            my $triggerTime = nanosToMillis($columns[$columnNumber]);
            my $startTime = $triggerTime - $preTime;
            my $interval = $preTime + $postTime;
            my $fileName = getFileName($serialNumber, $triggerTime);
            my $url = getURL($ip_address, $startTime, $interval, $serialNumber);
            my $ua = new LWP::UserAgent;
 
         my $response = $ua->get($url, ':content_file' => $fileName);
         if ($response->is_success) {
             my $msg = sprintf("Got file: %s\n", $fileName);
             print $msg;
         } else {
            my $msg = sprintf("Error for %s: %s\n", $fileName,
                $response->status_line);
            print $msg
         }
         $startTime = $startTime + $interval;
        }
    }
    close $in
}

# =========================
( $ip_address, $serialNumber, $csvFile, $preTime, $postTime ) = @ARGV;
my $preTimeInMs = durationInMillis($preTime);
my $postTimeInMs = durationInMillis($postTime);

getFiles($ip_address, $serialNumber, $csvFile, $preTimeInMs, $postTimeInMs);

