#!/usr/bin/perl 

# =========================
# Instructions:
# 1) Make sure Perl is installed.  If Perl is not download it.  
# For Windows download from:
# http://www.activestate.com/Products/ActivePerl/?_x=1 (Download the MSI package)
# For Linux/Solaris download from: http://www.perl.org
#
# 2) To run the script from a command line/shell prompt type:
# perl extract_time_onecomp.pl ip_address serial_number start_time duration interval
# 
# where ip_address is  the ip_address of the machine running apollo (can use localhost for ApolloLite)
# serial_number is the serial_number of the unit
# start_time is the start_time you want to get in format YYYY-MM-DD_HH:MM:SS
# duration is how many seconds in total to retreive
# interval is how many seconds long each file should be
# -includeIp is to include the IP Address in the file name
# =========================

# =========================
package NMX::Scripts;

use Time::Local;
use URI::URL;
use LWP::UserAgent;
use HTTP::Request;
use HTTP::Response;

# =========================
sub timeInMillis
{
    ( $inputTime ) = @_;

    $inputTime =~ s/(^\s*|\s*$)//g;
    if ($inputTime =~ /([0-9][0-9][0-9][0-9])[\-\/\.]([0-9][0-9])[\-\/\.]([0-9][0-9])[ _T\.]([0-9][0-9])[:\-\.]([0-9][0-9])[:\-\.]([0-9][0-9])($|\.[0-9]+)/)
    {
      my $value = timegm($6, $5, $4, $3, $2 - 1, $1);
      my $date = sprintf( "%04d-%02d-%02d_%02d:%02d:%02d", $1, $2, $3, $4, $5, $6 );
      $fraction =~ s/0\.//;
      $date = sprintf( "%s.%09d", $date, $fraction );
      my $valueMs = sprintf ("%d%03d", $value, $fraction / 1000000);
      return $valueMs
    }
    else
    {
      return 0;
    }
}

# =========================
sub durationInMillis
{
    ( $duration ) = @_;
    return $duration * 1000;
}

# =========================
sub getChannel
{
    ( $serialNumber, $channelnum ) = @_;
    $channel = sprintf ("taurus_%04d/band/timeSeries%d/", $serialNumber, $channelnum);
    return $channel;
}

# =========================
sub getURL
{
    ( $ip_address, $startTime, $duration, $serialNumber, $channel ) = @_;
    my $urlString =
    sprintf("http://%s/playback/download/?channels=%s&dataType=TimeSeries&dataFormat=MiniSEED&startMillis=%s&duration=%d s", $ip_address, $channel, $startTime, $duration / 1000);
    $url = URI::URL->new($urlString);
    return $url;
}
# =========================
sub getTimeString
{
    ( $inputTime ) = @_;
    $inputTime =~ s/(^\s*|\s*$)//g;
    $seconds = $inputTime / 1000;
    $fraction += $inputTime % 1000;
    $fraction /= 1000;
    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = gmtime($seconds);
    $mon++;
    $year += 1900;
    my $date = sprintf( "%02d-%02d-%02d_%02d_%02d_%02d",
        $year, $mon, $mday, $hour, $min, $sec );
    return $date;
}

# =========================
sub getFileName
{
    ( $serialNumber, $startTime, $num ) = @_;
    my $timeString = getTimeString($startTime);
    my $fileName = sprintf("taurus_%04d_%s_%s.seed", $serialNumber, $timeString, $num);
    if ( $includeIpInName =~ "-includeIp" )
    {
    	$fileName = $ip_address . "_" . $fileName;
    }
    return $fileName;
}

# =========================
sub getFiles 
{
    ( $ip_address, $serialNumber, $startTime, $duration, $interval ) = @_;
     
    $endTime = $startTime + $duration;
    while($startTime < $endTime){
        if ($startTime + $interval > $endTime) {
           $interval = $endTime - $startTime;
        }
        for (my $num = 1; $num < 4; $num++) {
			my $fileName = getFileName($serialNumber, $startTime, $num);
			my $channel = getChannel($serialNumber, $num + "");
			my $url = getURL($ip_address, $startTime, $interval, $serialNumber, $channel);
			my $ua = new LWP::UserAgent;
	
			my $response = $ua->get($url, ':content_file' => $fileName);
			if ($response->is_success) {
				my $msg = sprintf("Got file: %s\n", $fileName);
				print $msg;
			} else {
				print $response->error_as_HTML;
			}
        }
        $startTime = $startTime + $interval;
    }
}

# =========================
( $ip_address, $serialNumber, $startTime, $duration, $interval, $includeIpInName ) = @ARGV;
my $startTimeInMs = timeInMillis($startTime);
my $durationInMs = durationInMillis($duration);
my $intervalInMs = durationInMillis($interval);

getFiles($ip_address, $serialNumber, $startTimeInMs, $durationInMs, $intervalInMs);


__END__
:endofperl
